// OBJETOS

Adipico a; // molécula de ácido adípico, com 4 carbonos e uma carboxila em cada extremidade
Agua h; // molécula de água que é formada na união dos monômeros distintos
Diamina d; // molécula de hexametilenoamina, com 6 carbonos e um radical amino em cada extremidade
Polimero p; // porção de polímero que conta com objetos Diamina e Adipico unidos no formato de uma cadeia

//FONTE

PFont f;

// VARIÁVEIS GLOBAIS

float comprimento; // define a escala das moléculas
float velocidade; // velocidade da animação

// VARIÁVEIS DE ANIMAÇÃO

int etapa; // define a etapa da animação: em '0' a molécula de água ainda não se formou e os monômeros se aproximam,  
           // em '1' a molécula de água se afasta e os monômeros se unem,  e '2' o zoom out na cadeia polimérica
           
boolean play; // determina se há movimento ou não na 1ª parte da animação 

// BOTÕES

Botao botao;
Botao pausa;

// CRÉDITO

Credito credito;

// SETUP

void setup(){
  
 size(600, 600);
 smooth();
 strokeWeight(3); // a linha grossa transmite um ar 'amigável' à imagem das moléculas
 rectMode(CORNERS);
 textAlign(CENTER);
 //textMode(CENTER);
 
 // definição das variáveis globais
 comprimento = 60;
 velocidade = 0.6;
 
 // carregamos a fonte
 f = loadFont("CourierNewPS-BoldMT-28.vlw");
 textFont(f, 38);
 
 // inicializamos o botão que regula as etapas
 botao = new Botao(width/2 + 200, height/2 + 80, 80, 20, "PRÓXIMO");
 botao.situacao = "SELECIONADO";
 botao.psituacao = "LIBERADO";
 
 //inicializamos o botão de pausa
 pausa = new Botao(botao.xpos - 100, botao.ypos, 20, 20, ">");
 
 // definimos o crédito
 credito = new Credito();
 
 // inicialização
 a = new Adipico(width/2 - 3*comprimento/2 , height/2 - 50);
 d = new Diamina(width/2 + comprimento/2, a.ypos);
 h = new Agua(width/2 - comprimento/2, a.ypos);
 p = new Polimero(- comprimento, a.ypos);
 etapa = 0;
 play = false; 
 
 background(255); // ativamos 'background' dentro do SETUP pois vamos escrever o texto 
                  // apenas UMA vez
 
 //TEXTO
  fill(0);
  text("POLIMERIZAÇÃO:\nEtapas", width/2, height/11);
  textFont(f, 14);
    text("A polimerização por etapas ocorre pela interação química entre \n" +
         "os monômeros que gera, como resultado, a união entre os meros e a  \n" +
         "formação de moléculas pequenas que não fazem parte do polímero \n" +
         "(estas espécies condensam).\n \n" +
         "Acima acompanhamos, inicialmente, um passo da formação do nylon,\n" +
         "partindo do ácido adípico (acído dicarboxílico com cadeia de quatro\n" +
         "carbonos) e da hexametilenodiamina (diamina com seis carbonos),\n" +
         "formando uma molécula de água como subproduto da reação.\n\n" + 
         "Na etapa seguinte observamos uma porção da cadeia polimérica.", width/2, 3*height/4); 
}

//DRAW

void draw(){
  
  if(credito.botao.situacao != "LIBERADO"){
    background(255);
    fill(0);
    textFont(f, 38);
    text("POLIMERIZAÇÃO:\nEtapas", width/2, height/11);
    textFont(f, 14);
    text("A polimerização por etapas ocorre pela interação química entre \n" +
         "os monômeros que gera, como resultado, a união entre os meros e a  \n" +
         "formação de moléculas pequenas que não fazem parte do polímero \n" +
         "(estas espécies condensam).\n \n" +
         "Acima acompanhamos, inicialmente, um passo da formação do nylon,\n" +
         "partindo do ácido adípico (acído dicarboxílico com cadeia de quatro\n" +
         "carbonos) e da hexametilenodiamina (diamina com seis carbonos),\n" +
         "formando uma molécula de água como subproduto da reação.\n\n" + 
         "Na etapa seguinte observamos uma porção da cadeia polimérica.", width/2, 3*height/4);
  }
  
  fill(255);
  noStroke();
  rectMode(CORNERS);
  rect(0, a.ypos - height/4 - 5, width, a.ypos + height/4 + 10); // para não apagar o texto e, mesmo assim, atualizarmos a animação,
                                                             // ao invés de colorir toda a tela, definimos um retângulo que será
                                                             // apagado e coberto a cada passo da animação.
  roda(); // rodamos a animação
  legenda(width/20, 3*height/5);
  botao.display();
  pausa.display();
  credito.display();
}

// função de animação
void roda(){
  
  if(etapa < 2){
    //desenhamos as moéculas de ác. adípico e da diamina
    d.display(); 
    a.display();
  
    if(play){
      // se a animação estiver rodando, as moléculas se preparam para a união: 'abrem-se'
      a.abre(velocidade/24);
      d.abre(velocidade/24);
  
      // enquanto as moléculas não estiverem perto o suficiente (a água ainda não se formou), a molécula de
      // ác. adípico vai se aproximando da outra
      if(d.xpos - a.xpos > 3*comprimento/2 ){
      a.anda(velocidade, 0);
      }  
      else
        etapa = 1; //se a molécula de água está formada, define-se a etapa 1
  
    }
    
      if(etapa == 1){
        
        // enquanto os monômeros ainda não estão unidos, eles se aproximam
        if(d.xpos - a.xpos > comprimento/2 && play == true){
          a.anda(velocidade, 0);
          d.anda(- velocidade, 0);
        }
        
        // à medida que a molécula de água vai se distanciando
       if(h.ypos < a.ypos + height/6){
        h.display(); // desenhamos a molécula de água
        if(play){
          h.anda(0, 3*velocidade/2); // ela anda para a parte inferior da tela
          h.gira(0.005); // e gira sutilmente
        }
       }
      }
  }
  
  // na segunda etapa mostramos o polímero montado e diminuimos progressivamente a escala,
  // dando a impressão de que a câmera se afasta da cadeia polimérica
  if(etapa == 2){
    
    if(play)
      p.update(); // atualiza a escala do polímero
    
    p.display(); // desenha o polímero
    
    if(play){
      if(comprimento > 20){
      
        if(comprimento < 50 && comprimento > 30)
          strokeWeight(2);
      
        if(comprimento < 30)
          strokeWeight(1); 
      
        comprimento -= 0.1; // diminuição da escala até comprimento atingir o valor 20
      }
    }
  }
}
 // escrevemos a legenda de cores, relacionando os átomos
void legenda(float x, float y){
  textAlign(LEFT);
  textFont(f, 12);
  rectMode(CENTER);
  pushMatrix();
  translate(x, y);
  fill(255);
  rect(70, 30, 140, 80);
  fill(0);
  noStroke();
  text("Legenda:", 0, 0);
  text("Carbono", 60, 15);
  fill(#91D6F7); //AZUL
  ellipse(50, 11, 10, 10);
  fill(0);
  text("Hidrogênio", 60, 30);
  fill(122);
  ellipse(50, 26, 10, 10);
  fill(0);
  text("Nitrogênio", 60, 45);
  fill(#08FF85);
  ellipse(50, 41, 10, 10);
  fill(0);
  text("Oxigênio", 60, 60);
  fill(#FC928F);
  ellipse(50, 56, 10, 10);
  popMatrix();
  textAlign(CENTER);
  rectMode(CORNERS);
}

// ao apertar o mouse
void mousePressed(){
  
  if(pausa.situacao == "SELECIONADO" || etapa == 2)
    botao.mousePressed();
    
  pausa.mousePressed();
  // no passo da polimerização o clique pode 'pausar' a animação
  //if(etapa < 2){
  if(botao.situacao == "PRESSIONADO"){ 
   /* 
    if(pausa.situacao == "LIBERADO")
      play = true;
    else if(pausa.situacao == "SELECIONADO")
      play = false;
    */
    // na apresentação do polímero, o clique reinicia a animação  
    if(etapa == 2){
     strokeWeight(3);    
     comprimento = 60;
     etapa = 0;
     a = new Adipico(width/2 - 3*comprimento/2 , height/2 - 20);
     d = new Diamina(width/2 + comprimento/2, a.ypos);
     h = new Agua(width/2 - comprimento/2, a.ypos);
     p = new Polimero(- comprimento, a.ypos);
     play = false;
     pausa.psituacao = "SELECIONADO";
     pausa.situacao = "LIBERADO";
     pausa.texto = ">";
     botao.texto = "PRÓXIMO";
   }
   
    // durante a etapa '1', o clique muda para a etapa da animação
    if(etapa == 1){
      etapa ++;
      botao.texto = "INÍCIO";
    }
     
      
  }
 
   
  
  
  credito.botao.mousePressed();
}

void mouseReleased(){
  
 /* if(botao.situacao == "PRESSIONADO")
    botao.libera();*/
  
  boolean pressionado = false;
  
  if(pausa.situacao == "PRESSIONADO")
    pressionado = true;
  
  pausa.mouseReleased();
  
  if(pausa.situacao == "LIBERADO" && pressionado){
    play = false;
    pausa.texto = ">";
  }
  else if(pausa.situacao == "SELECIONADO" && pressionado){
    play = true;
    pausa.texto = "||";
  }
  
  if(pausa.situacao == "SELECIONADO")
    botao.libera();
    
  credito.botao.mouseReleased();
  
}

